#include "branchitem.h"
#include "branchobj.h"
#include "vymmodel.h"

#include <iostream>
#include <QDir>

using namespace std;

BranchItem::BranchItem(const QList<QVariant> &data, TreeItem *parent):TreeItem (data,parent)
{
	//cout << "Constr. BranchItem\n";

	scrolled=false;
	tmpUnscrolled=false;
}

BranchItem::~BranchItem()
{
	cout << "Destr. BranchItem\n";
    qDeleteAll(childItems);
}

void BranchItem::copy (BranchItem *other)
{
	scrolled=other->scrolled;
	tmpUnscrolled=other->tmpUnscrolled;
}

QString BranchItem::saveToDir (const QString &tmpdir,const QString &prefix, const QPointF& offset)
{
	// Cloudy stuff can be hidden during exports
	if (hidden) return QString();

    QString s,a;
	BranchObj *bo=(BranchObj*)lmo;

	/* FIXME-1
	// Update of note is usually done while unselecting a branch
	if (isNoteInEditor) getNoteFromTextEditor();
	
	QString scrolledAttr;
	if (scrolled) 
		scrolledAttr=attribut ("scrolled","yes");
	else
		scrolledAttr="";

	// save area, if not scrolled
	QString areaAttr;
	if (!((BranchObj*)(parObj))->isScrolled() )
	{
		areaAttr=
			attribut("x1",QString().setNum(absPos.x()-offset.x())) +
			attribut("y1",QString().setNum(absPos.y()-offset.y())) +
			attribut("x2",QString().setNum(absPos.x()+width()-offset.x())) +
			attribut("y2",QString().setNum(absPos.y()+height()-offset.y()));

	} else
		areaAttr="";
	
	// Providing an ID for a branch makes export to XHTML easier
	QString idAttr;
	if (countXLinks()>0)
		idAttr=attribut ("id",model->getSelectString(this)); //TODO directly access model
	else
		idAttr="";

	*/
    s=beginElement ("branch" 
	//	+getOrnXMLAttr() 
	//	+scrolledAttr 
	//	+areaAttr 
	//	+idAttr 
	//	+getIncludeImageAttr() 
		);
    incIndent();

	// save heading
    s+=valueElement("heading", getHeading(),
		attribut ("textColor",QColor( bo->getColor()).name()));

/*
	// Save frame
	if (frame->getFrameType()!=FrameObj::NoFrame) 
		s+=frame->saveToDir ();

	// save names of flags set
	s+=standardFlags->saveToDir(tmpdir,prefix,0);
	
	// Save FloatImages
	for (int i=0; i<floatimage.size(); ++i)
		s+=floatimage.at(i)->saveToDir (tmpdir,prefix);
*/

	// save note
	if (!note.isEmpty() )
		s+=note.saveToDir();
	
	// Save branches
	TreeItem *ti=getFirstBranch();
	TreeItem *last=getLastBranch();
	while (ti && ti!=last) 
		s+=ti->saveToDir(tmpdir,prefix,offset);

	/*
	// Save XLinks
	QString ol;	// old link
	QString cl;	// current link
	for (int i=0; i<xlink.size(); ++i)
	{
		cl=xlink.at(i)->saveToDir();
		if (cl!=ol)
		{
			s+=cl;
			ol=cl;
		} else
		{
			qWarning (QString("Ignoring of duplicate xLink in %1").arg(getHeading()));
		}
	}	
	*/

    decIndent();
    s+=endElement   ("branch");
    return s;
}

void BranchItem::updateVisibility()	// FIXME-1
{
}

void BranchItem::setVymLink(QString s)
{
	if (!s.isEmpty())
	{
		// We need the relative (from loading) 
		// or absolute path (from User event)
		// and build the absolute path.
		// Note: If we have relative, use path of
		// current map to build absolute path
		QDir d(s);
		if (!d.path().startsWith ("/"))
		{
			QString p=model->getDestPath();
			int i=p.findRev("/",-1);
			d.setPath(p.left(i)+"/"+s);
			d.convertToAbs();
		}
		vymLink=d.path();
		//FIXME-2 systemFlags->activate("vymLink");
	}	
	else	
	{
		//FIXME-2 systemFlags->deactivate("vymLink");
		vymLink="";
	}	
	/* FIXME-2
	calcBBoxSize();			// recalculate bbox
    positionBBox();			// rearrange contents
	forceReposition();
	*/
}

QString BranchItem::getVymLink()
{
	return vymLink;
}

void BranchItem::unScroll()
{
	if (tmpUnscrolled) resetTmpUnscroll();
	if (scrolled) toggleScroll();
}

void BranchItem::toggleScroll()
{
	if (scrolled)
	{
		scrolled=false;
		//FIXME-1 systemFlags->deactivate("scrolledright");
		/*
		for (int i=0; i<branch.size(); ++i)
			branch.at(i)->setVisibility(true);
		*/	
	} else
	{
		scrolled=true;
		/*
		//FIXME-1 systemFlags->activate("scrolledright");
		for (int i=0; i<branch.size(); ++i)
			branch.at(i)->setVisibility(false);
		*/	
	}
	/*	
	calcBBoxSize();
	positionBBox();	
	move (absPos.x(), absPos.y() );
	forceReposition();
	*/
}

bool BranchItem::isScrolled()
{
	return scrolled;
}

bool BranchItem::hasScrolledParent(BranchItem *start)
{
	// Calls parents recursivly to
	// find out, if we are scrolled at all.
	// But ignore myself, just look at parents.

	if (this !=start && scrolled) return true;

	BranchItem* bi=(BranchItem*)parentItem;
	if (bi) 
		return bi->hasScrolledParent(start);
	else
		return false;
}

void BranchItem::tmpUnscroll()
{
	// Unscroll parent (recursivly)
	BranchItem * bi=(BranchItem*)parentItem;
	if (bi) bi->tmpUnscroll();
		
	// Unscroll myself
	if (scrolled)
	{
		tmpUnscrolled=true;
		// FIXME-1 systemFlags->activate("tmpUnscrolledright");
		toggleScroll();
	}	
}

void BranchItem::resetTmpUnscroll()
{
	// Unscroll parent (recursivly)
	BranchItem * bi=(BranchItem*)parentItem;
	if (bi) bi->resetTmpUnscroll();
		
	// Unscroll myself
	if (tmpUnscrolled)
	{
		tmpUnscrolled=false;
		// FIXME-1 systemFlags->deactivate("tmpUnscrolledright");
		toggleScroll();
	}	
}

TreeItem* BranchItem::findMapItem (QPointF p, TreeItem* excludeLMO)
{
	// Search branches
	TreeItem *ti;
	for (int i=0; i<branchCount(); ++i)
    {	
		ti=getBranchNum(i)->findMapItem(p, excludeLMO);
		if (ti != NULL) return ti;
    }
	

	// Search myself
    if (getBranchObj()->isInClickBox (p) && (this != excludeLMO) && getBranchObj()->isVisibleObj() ) 
		return this;

/* FIXME-2 // Search float images
    for (int i=0; i<floatimage.size(); ++i )
		if (floatimage.at(i)->isInClickBox(p) && 
			(floatimage.at(i) != excludeLMO) && 
			floatimage.at(i)->getParObj()!= excludeLMO &&
			floatimage.at(i)->isVisibleObj() 
		) return floatimage.at(i)->getTreeItem();
*/
    return NULL;
}

TreeItem* BranchItem::findID (QString sid)
{
	// Search branches
    TreeItem *ti;
	for (int i=0; i<branchCount(); ++i)
    {	
		ti=getBranchNum(i)->findID (sid);
		if (ti != NULL) return ti;
    }
	
	// Search myself
	if (sid==objID) return this;


/*
	// Search float images
    for (int i=0; i<floatimage.size(); ++i )
		if (floatimage.at(i)->inBox(p) && 
			(floatimage.at(i) != excludeLMO) && 
			floatimage.at(i)->getParObj()!= excludeLMO &&
			floatimage.at(i)->isVisibleObj() 
		) return floatimage.at(i);
*/
    return NULL;
}


BranchObj* BranchItem::getBranchObj()	// FIXME-3 only for transition BO->BI
{
	return (BranchObj*)lmo;
}

